==> http_libs-1.0.1-sources.jar: com/adups/http_libs/HttpManager.java <==
package com.adups.http_libs;

import android.content.Context;
import android.os.AsyncTask;

import com.adups.trace.Trace;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.net.ssl.HttpsURLConnection;

/**
 * Created by fighter_lee on 2017/7/18.
 */

public class HttpManager {

    private static final String TAG = HttpManager.class.getSimpleName();
    private static final String HEADER_CONTENT_TYPE = "Content-Type";
    private static final String HEADER_CONTENT_LENGTH = "Content-Length";
    private HttpConfig config;

    public static HttpConfig build(Context context) {
        return new HttpConfig(context);
    }

    protected HttpManager(HttpConfig config) {
        initConfig(config);
    }

    private void initConfig(HttpConfig config) {
        this.config = config;
        Trace.d(TAG, "initConfig() " + config.toString());
    }

    public final HttpConfig getConfig() {
        return config;
    }

    public <T> void executeAsync(AbstractRequest<T> request) {
        ExecuteAsyncTask executeAsyncTask = new ExecuteAsyncTask();
        executeAsyncTask.execute(request);
    }

    private class ExecuteAsyncTask extends AsyncTask<AbstractRequest, Integer, InternalResponse> {

        @Override
        protected void onPreExecute() {
            super.onPreExecute();
        }

        @Override
        protected InternalResponse doInBackground(AbstractRequest[] abstractRequests) {
            AbstractRequest abstractRequest = abstractRequests[0];
            InternalResponse response = handleRequest(abstractRequest);
            HttpListener httpListener = abstractRequest.getHttpListener();
            response.setHttpListener(httpListener);

            if (httpListener != null) {
                httpListener.notifyCallStart(abstractRequest);
            }

            try {
                connectWithRetry(abstractRequest, response);
            } catch (HttpException e) {
                e.printStackTrace();
                response.setException(e);
                if (httpListener != null) {
                    httpListener.notifyCallFailure(e, response);
                }
            } catch (IOException e) {
                e.printStackTrace();
                HttpException httpException = new HttpException(e);
                response.setException(httpException);
                if (httpListener != null) {
                    httpListener.notifyCallFailure(httpException, response);
                }
            } finally {
                return response;
            }
        }

        @Override
        protected void onPostExecute(InternalResponse internalResponse) {
            HttpListener listener = internalResponse.getListener();
            listener.notifyCallEnd(internalResponse);
            super.onPostExecute(internalResponse);
        }
    }

    private void connectWithRetry(AbstractRequest abstractRequest, InternalResponse response) throws HttpException, IOException {

        response.setRedirectTimes(0);//初始化重定向次数
        response.setRetryTimes(0);//初始化重试次数
        for (int i = 1; i <= abstractRequest.getMaxRetryTimes(); i++) {
            if (response.getRetryTimes() > 0) {
                response.getListener().notifyCallRetry(abstractRequest, abstractRequest.getMaxRetryTimes(), response.getRetryTimes());
            }
            boolean connect = connect(abstractRequest, response);
            Trace.d(TAG, "connectWithRetry() " + connect);
            if (connect) {
                break;//成功则跳出
            } else {
                response.setRetryTimes(response.getRetryTimes() + 1);
            }
        }
    }

    private boolean connect(AbstractRequest abstractRequest, InternalResponse response) throws HttpException, IOException {
        String u = abstractRequest.getUrl();
        URL url = null;
        HttpURLConnection connection;
        InputStream inputStream = null;
        try {
            url = new URL(u);
            if (u.startsWith("https")) {
                connection = (HttpsURLConnection) url.openConnection();
            } else {
                connection = (HttpURLConnection) url.openConnection();
            }
            connection.setConnectTimeout(abstractRequest.getConnectTimeout());
            connection.setReadTimeout(abstractRequest.getSocketTimeout());
            connection.setRequestMethod(abstractRequest.getMethod().getMethodName());
            connection.setDoInput(true);
            connection.setDoOutput(true);
            connection.setRequestProperty(HEADER_CONTENT_TYPE, abstractRequest.getContentType().getType());

            byte[] request_data = abstractRequest.getContent();
            if (null != request_data) {
                connection.setFixedLengthStreamingMode(request_data.length);
                connection.setRequestProperty(HEADER_CONTENT_LENGTH,
                        String.valueOf(request_data.length));
                OutputStream outputStream = connection.getOutputStream();
                outputStream.write(request_data);
            }

            inputStream = connection.getInputStream();

            if (inputStream == null) {
                throw new HttpException(HttpException.HTTP_ERROR_UNREACHABLE);
            }

            int statusCode = connection.getResponseCode();

            HttpStatus httpStatus = new HttpStatus(statusCode, connection.getResponseMessage());
            response.setHttpStatus(httpStatus);

            ArrayList<NameValuePair> headers = new ArrayList<NameValuePair>();
            for (Map.Entry<String, List<String>> header : connection.getHeaderFields().entrySet()) {
                if (header.getKey() != null) {
                    List<String> values = header.getValue();
                    if (values != null) {
                        for (String value : values) {
                            headers.add(new NameValuePair(header.getKey(), value));
                        }
                    }
                }
            }
            response.setHeaders(headers);
            response.setContentLength(connection.getContentLength());
            response.setContentEncoding(connection.getContentEncoding());
            response.setContentType(connection.getContentType());

            if (statusCode <= 299 || statusCode == 600) {
                String charSet = getCharsetByContentType(response.getContentType(), abstractRequest.getCharSet());
                response.setCharSet(charSet);
                String result = getResultString(inputStream, abstractRequest.getCharSet());
                Data data = new Data().setApi(u).setMessage(result);
                response.setData(data);
                abstractRequest.getHttpListener().notifyCallSuccess(data, response);
                return true;

            } else if (statusCode <= 399) {
                //重定向
                if (response.getRedirectTimes() <= abstractRequest.getMaxRedirectTimes()) {
                    //重新请求
                    abstractRequest.getHttpListener().notifyCallRedirect(
                            abstractRequest, abstractRequest.getMaxRedirectTimes(), response.getRedirectTimes());
                    response.setRedirectTimes(response.getRedirectTimes() + 1);
                    connect(abstractRequest, response);
                }

            } else if (statusCode <= 499) {
                // 客户端被拒
                throw new HttpException(httpStatus.getCode(), new Throwable(httpStatus.getDescriptionInChinese()));
            } else if (statusCode < 599) {
                // 服务器有误
                throw new HttpException(httpStatus.getCode(), new Throwable(httpStatus.getDescriptionInChinese()));
            }
        } finally {
            if (inputStream != null) {
                inputStream.close();
            }
        }
        return false;
    }

    protected <T> InternalResponse<T> handleRequest(AbstractRequest<T> request) {
        //设置默认的数据
        if (request.getConnectTimeout() <= 0) {
            request.setConnectTimeout(config.getConnectTimeout());
        }
        if (request.getSocketTimeout() <= 0) {
            request.setSocketTimeout(config.getSocketTimeout());
        }
        if (request.getMethod() == null) {
            request.setMethod(HttpMethods.Get);
        }
        if (request.getCharSet() == null) {
            request.setCharSet(config.DEFAULT_CHARSET);
        }
        if (request.getMaxRetryTimes() <= 0) {
            request.setMaxRetryTimes(config.getRetryTimes());
        }
        if (request.getMaxRedirectTimes() <= 0) {
            request.setMaxRedirectTimes(config.getRedirectTimes());
        }
        if (request.getContentType() == null) {
            request.setHeaderContentType(config.CONTENT_TYPE);
        }

        return new InternalResponse<T>(request);
    }

    private String getCharsetByContentType(String contentType, String defCharset) {
        if (contentType != null) {
            String[] values = contentType.split(";"); // values.length should be 2
            for (String value : values) {
                value = value.trim();
                if (value.toLowerCase().startsWith("charset=")) {
                    return value.substring("charset=".length());
                }
            }
        }
        return defCharset == null ? HttpConfig.DEFAULT_CHARSET : defCharset;
    }

    private static String getResultString(InputStream inputStream, String encode) {
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        byte[] data = new byte[1024];
        int len = 0;
        String result = "";
        if (inputStream != null) {
            try {
                while ((len = inputStream.read(data)) != -1) {
                    outputStream.write(data, 0, len);
                }
                result = new String(outputStream.toByteArray(), encode);

            } catch (IOException e) {
                Trace.d(TAG, "getResultString() exception:" + e.toString());
                e.printStackTrace();
            }
        }
        return result;
    }
}

==> http_libs-1.0.2-sources.jar: com/adups/http_libs/HttpManager.java <==
package com.adups.http_libs;

import android.content.Context;
import android.os.AsyncTask;

import com.adups.trace.Trace;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.net.ssl.HttpsURLConnection;

/**
 * Created by fighter_lee on 2017/7/18.
 */

public class HttpManager {

    private static final String TAG = HttpManager.class.getSimpleName();
    private static final String HEADER_CONTENT_TYPE = "Content-Type";
    private static final String HEADER_CONTENT_LENGTH = "Content-Length";
    private HttpConfig config;

    public static HttpConfig build(Context context) {
        return new HttpConfig(context);
    }

    protected HttpManager(HttpConfig config) {
        initConfig(config);
    }

    private void initConfig(HttpConfig config) {
        this.config = config;
        Trace.d(TAG, "initConfig() " + config.toString());
    }

    public final HttpConfig getConfig() {
        return config;
    }

    /**
     * 异步请求
     * @param request
     * @param <T>
     */
    public <T> void enqueue(AbstractRequest<T> request) {
        ExecuteAsyncTask executeAsyncTask = new ExecuteAsyncTask();
        executeAsyncTask.execute(request);
    }

    /**
     * 同步请求
     * @param request
     * @param <T>
     * @return
     */
    public<T> InternalResponse execute(AbstractRequest<T> request){
        InternalResponse response = handleRequest(request);
        if (request.getHttpListener() != null){
            //取消回调
            request.getHttpListener().disableListener(true);
        }
        connectWithRetry(request,response);
        return response;
    }

    private class ExecuteAsyncTask extends AsyncTask<AbstractRequest, Integer, InternalResponse> {

        @Override
        protected void onPreExecute() {
            super.onPreExecute();
        }

        @Override
        protected InternalResponse doInBackground(AbstractRequest[] abstractRequests) {
            AbstractRequest abstractRequest = abstractRequests[0];
            InternalResponse response = handleRequest(abstractRequest);
            HttpListener httpListener = abstractRequest.getHttpListener();

            if (httpListener != null) {
                httpListener.notifyCallStart(abstractRequest);
            }

            boolean b = connectWithRetry(abstractRequest, response);
            if (!b) {
                if (httpListener != null) {
                    httpListener.notifyCallFailure(response.getException(), response);
                }
            }
            return response;
        }

        @Override
        protected void onPostExecute(InternalResponse internalResponse) {
            HttpListener listener = internalResponse.getRequest().getHttpListener();
            listener.notifyCallEnd(internalResponse);
            super.onPostExecute(internalResponse);
        }
    }

    private boolean connectWithRetry(AbstractRequest abstractRequest, InternalResponse response) {

        response.setRedirectTimes(0);//初始化重定向次数
        response.setRetryTimes(0);//初始化重试次数
        for (int i = 1; i <= abstractRequest.getMaxRetryTimes() + 1; i++) {
            if (i > 1) {
                //第二次请求回调重试
                response.setRetryTimes(response.getRetryTimes() + 1);
                if (abstractRequest.getHttpListener() != null) {
                    abstractRequest.getHttpListener().notifyCallRetry(abstractRequest, abstractRequest.getMaxRetryTimes(), response.getRetryTimes());
                }
            }
            boolean connect = false;
            try {
                connect = connect(abstractRequest, response);
            } catch (HttpException e) {
                e.printStackTrace();
                response.setException(e);
            } catch (IOException e) {
                e.printStackTrace();
                HttpException httpException = new HttpException(e);
                response.setException(httpException);
            }
            if (connect) {
                return true;
            }
        }
        return false;
    }

    private boolean connect(AbstractRequest abstractRequest, InternalResponse response) throws HttpException, IOException {
        Trace.d(TAG, "connect() start.");
        String u = abstractRequest.getUrl();
        URL url = null;
        HttpURLConnection connection;
        InputStream inputStream = null;
        try {
            url = new URL(u);
            if (u.startsWith("https")) {
                connection = (HttpsURLConnection) url.openConnection();
            } else {
                connection = (HttpURLConnection) url.openConnection();
            }
            connection.setConnectTimeout(abstractRequest.getConnectTimeout());
            connection.setReadTimeout(abstractRequest.getSocketTimeout());
            connection.setRequestMethod(abstractRequest.getMethod().getMethodName());
            connection.setDoInput(true);
            connection.setDoOutput(true);
            connection.setRequestProperty(HEADER_CONTENT_TYPE, abstractRequest.getContentType().getType());

            byte[] request_data = abstractRequest.getContent();
            if (null != request_data) {
                connection.setFixedLengthStreamingMode(request_data.length);
                connection.setRequestProperty(HEADER_CONTENT_LENGTH,
                        String.valueOf(request_data.length));
                OutputStream outputStream = connection.getOutputStream();
                outputStream.write(request_data);
            }

            inputStream = connection.getInputStream();

            if (inputStream == null) {
                throw new HttpException(HttpException.HTTP_ERROR_UNREACHABLE);
            }

            int statusCode = connection.getResponseCode();

            HttpStatus httpStatus = new HttpStatus(statusCode, connection.getResponseMessage());
            response.setHttpStatus(httpStatus);

            ArrayList<NameValuePair> headers = new ArrayList<NameValuePair>();
            for (Map.Entry<String, List<String>> header : connection.getHeaderFields().entrySet()) {
                if (header.getKey() != null) {
                    List<String> values = header.getValue();
                    if (values != null) {
                        for (String value : values) {
                            headers.add(new NameValuePair(header.getKey(), value));
                        }
                    }
                }
            }
            response.setHeaders(headers);
            response.setContentLength(connection.getContentLength());
            response.setContentEncoding(connection.getContentEncoding());
            response.setContentType(connection.getContentType());

            if (statusCode <= 299 || statusCode == 600) {
                String charSet = getCharsetByContentType(response.getContentType(), abstractRequest.getCharSet());
                response.setCharSet(charSet);
                String result = getResultString(inputStream, abstractRequest.getCharSet());
                Data data = new Data().setApi(u).setMessage(result);
                response.setData(data);
                if (abstractRequest.getHttpListener() != null) {
                    abstractRequest.getHttpListener().notifyCallSuccess(data, response);
                }
                return true;

            } else if (statusCode <= 399) {
                //重定向
                if (response.getRedirectTimes() < abstractRequest.getMaxRedirectTimes()) {
                    //重新请求
                    response.setRedirectTimes(response.getRedirectTimes() + 1);
                    if (abstractRequest.getHttpListener() != null) {
                        abstractRequest.getHttpListener().notifyCallRedirect(
                                abstractRequest, abstractRequest.getMaxRedirectTimes(), response.getRedirectTimes());
                    }
                    connect(abstractRequest, response);
                }

            } else if (statusCode <= 499) {
                // 客户端被拒
                throw new HttpException(httpStatus.getCode(), new Throwable(httpStatus.getDescriptionInChinese()));
            } else if (statusCode < 599) {
                // 服务器有误
                throw new HttpException(httpStatus.getCode(), new Throwable(httpStatus.getDescriptionInChinese()));
            }
        } finally {
            if (inputStream != null) {
                inputStream.close();
            }
        }
        return false;
    }

    protected <T> InternalResponse<T> handleRequest(AbstractRequest<T> request) {
        //设置默认的数据
        if (request.getConnectTimeout() <= 0) {
            request.setConnectTimeout(config.getConnectTimeout());
        }
        if (request.getSocketTimeout() <= 0) {
            request.setSocketTimeout(config.getSocketTimeout());
        }
        if (request.getMethod() == null) {
            request.setMethod(HttpMethods.Get);
        }
        if (request.getCharSet() == null) {
            request.setCharSet(config.DEFAULT_CHARSET);
        }
        if (request.getMaxRetryTimes() <= 0) {
            request.setMaxRetryTimes(config.getRetryTimes());
        }
        if (request.getMaxRedirectTimes() <= 0) {
            request.setMaxRedirectTimes(config.getRedirectTimes());
        }
        if (request.getContentType() == null) {
            request.setHeaderContentType(config.CONTENT_TYPE);
        }

        return new InternalResponse<T>(request);
    }

    private String getCharsetByContentType(String contentType, String defCharset) {
        if (contentType != null) {
            String[] values = contentType.split(";"); // values.length should be 2
            for (String value : values) {
                value = value.trim();
                if (value.toLowerCase().startsWith("charset=")) {
                    return value.substring("charset=".length());
                }
            }
        }
        return defCharset == null ? HttpConfig.DEFAULT_CHARSET : defCharset;
    }

    private static String getResultString(InputStream inputStream, String encode) {
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        byte[] data = new byte[1024];
        int len = 0;
        String result = "";
        if (inputStream != null) {
            try {
                while ((len = inputStream.read(data)) != -1) {
                    outputStream.write(data, 0, len);
                }
                result = new String(outputStream.toByteArray(), encode);

            } catch (IOException e) {
                Trace.d(TAG, "getResultString() exception:" + e.toString());
                e.printStackTrace();
            }
        }
        return result;
    }
}
